#import "MultiGetterDelegate.h"
#import "MultiGetterConcatenator.h"
#import <ncurses.h>
#import <sys/stat.h>
	
@implementation MultiGetterDelegate
	
- (id)initWithTag: (int)aTag concatenator: (id)cat outputDir: (NSString *)dir;
{
	[super init];
	tag = aTag;
	concatenator = cat;
	outputDir = [dir retain];
	downloaded = 0;
	length = 0;
	prevPercent = -1;
	begin = NO;
	return self;
}

- (void)downloadDidBegin:(NSURLDownload *)download
{
	//printf("hoge\n");
}

- (void) download : (NSURLDownload *) dl didReceiveResponse : (NSURLResponse *) response
{
	length = [response expectedContentLength];
}

- (void) download : (NSURLDownload *) dl decideDestinationWithSuggestedFilename : (NSString *) filename
{
	//printf("%s\n",[filename cString]);
	NSString *dst = [outputDir stringByAppendingPathComponent: [NSString stringWithFormat:@"%@_segments", filename]];
	mkdir([dst UTF8String], 0755);
	[dl setDestination: [dst stringByAppendingPathComponent: [NSString stringWithFormat:@"%@.%d", filename, tag]]  allowOverwrite: YES];
	[concatenator setPath: [outputDir stringByAppendingPathComponent: [NSString stringWithFormat: @"%@", filename]]];
}

- (void) download : (NSURLDownload *) dl didCreateDestination : (NSString *) asDstPath
{
	struct timeval tv;
	gettimeofday(&tv, NULL);
	beginTime = tv.tv_sec+(double)tv.tv_usec/1000000.0f;
	begin = YES;
}

- (BOOL)download:(NSURLDownload *)download shouldDecodeSourceDataOfMIMEType:(NSString *)encodingType
{
	return NO;
}

- (void) download : (NSURLDownload *) dl didReceiveDataOfLength : (unsigned) len
{
	/*
	int i, n=0, percent;
	char progress[32];
	float speed;
	*/
	downloaded += len;
	/*
	percent = (int)(100.0*downloaded/length);
	if(percent == prevPercent) return;
	prevPercent = percent;
	progress[n++] = '|';
	for(i=0;i<20;i++) {
		if(percent/5 > i)
			progress[n++] = '=';
		else if(percent/5 == i)
			progress[n++] = '>';
		else progress[n++] ='-';
	}
	progress[n++] = '|';
	progress[n++] = '\0';
	int elapsed = time(NULL)-beginTime;
	if(elapsed != 0) speed = (float)downloaded/elapsed/1024;
	else speed = 0;
	mvprintw(tag+2, 0, "   %3d  %s %3d%% [%d/%d]", tag, progress, percent, downloaded, length);
	mvprintw(tag+2, 65, "%6.1f KB/s", speed);
	refresh();
	*/
}

- (int)printCurrentProgress
{
	if(!begin) return downloaded;
	int i, n=0, percent;
	char progress[32];
	float speed;
	struct timeval tv;
	
	percent = (int)(100.0*downloaded/length);
	prevPercent = percent;
	if(percent == 100) begin = NO;
	progress[n++] = '|';
	for(i=0;i<20;i++) {
		if(percent/5 > i)
			progress[n++] = '=';
		else if(percent/5 == i)
			progress[n++] = '>';
		else progress[n++] ='-';
	}
	progress[n++] = '|';
	progress[n++] = '\0';
	gettimeofday(&tv, NULL);
	float elapsed = tv.tv_sec+(double)tv.tv_usec/1000000.0f-beginTime;
	if(elapsed < 0.5) speed = 0;
	else speed = (float)downloaded/elapsed/1024;
	mvprintw(tag+2, 0, "   %3d  %s %3d%% [%d/%d]", tag, progress, percent, downloaded, length);
	mvprintw(tag+2, 65, "%6.1f KB/s", speed);
	return downloaded;
}

- (void) downloadDidFinish : (NSURLDownload *) dl
{
	//mvprintw(tag+2, 0, "segment %d: done.\n", tag);
	[concatenator decreaseSegment];
}

- (void) download : (NSURLDownload *) dl didFailWithError : (NSError *) err
{
	endwin();
	printf("download failed(segment %d): %s\n", tag, [[err localizedDescription] cString]);
	exit(0);
}

@end