#import "MultiGetterFilesizeDecider.h"
#import "MultiGetterDelegate.h"
#import "MultiGetterConcatenator.h"
#import <ncurses.h>
#import <sys/stat.h>
	
@implementation MultiGetterFilesizeDecider
	
- (id)initWithSegmentNum: (int)seg outputDir: (NSString *)dir
{
	[super init];
	segment = seg;
	outputDir = [dir retain];
	referer = nil;
	arr = [[NSMutableArray alloc] init];
	return self;
}

- (void)setReferer: (NSString *)str
{
	referer = [str retain];
}

- (void) download : (NSURLDownload *) dl didReceiveResponse : (NSURLResponse *) response
{
	initscr();
	mvprintw(0, 0, "downloading %s...\n", [[[response URL] absoluteString] cString]);
	mvprintw(1, 0, "seg. #  progress                                                 speed");
	mvprintw(2, 0, "------  -------------------------------------------------------  ------------");
	refresh();
	int i;
	struct timeval tv;
	length = [response expectedContentLength];
	int segmentSize = length / segment;
	MultiGetterConcatenator *cat = [[MultiGetterConcatenator alloc] initWithSegmentNum: segment];
	//printf("%d, %d\n",len, segmentSize);
	for(i=0;i<segment;i++) {
		NSMutableURLRequest *req = [NSMutableURLRequest requestWithURL: [[dl request] URL] cachePolicy: NSURLRequestUseProtocolCachePolicy timeoutInterval: 120 ];
		if(referer) [req setValue: referer forHTTPHeaderField: @"Referer"];
		if(i!=segment-1) [req setValue: [NSString stringWithFormat: @"bytes=%d-%d",i*segmentSize, i*segmentSize+segmentSize-1] forHTTPHeaderField: @"Range"];
		else [req setValue: [NSString stringWithFormat: @"bytes=%d-",i*segmentSize] forHTTPHeaderField: @"Range"];
		//[req setValue: @"Keep-Alive" forHTTPHeaderField: @"Connection"];
		//[req setValue: @"timeout=120, max=99" forHTTPHeaderField: @"Keep-Alive"];
		MultiGetterDelegate *del = [[MultiGetterDelegate alloc] initWithTag: i+1 concatenator: cat outputDir: outputDir];
		[[NSURLDownload alloc] initWithRequest: req delegate: del];
		[arr addObject: del];
		[cat increaseSegment];
	}
	[dl cancel];
	gettimeofday(&tv, NULL);
	beginTime = tv.tv_sec+(double)tv.tv_usec/1000000.0f;
	NSTimer *timer = [NSTimer scheduledTimerWithTimeInterval:0.3 target:self selector:@selector(printProgress) userInfo:self repeats:YES];
	[cat addTimer: timer];
}

- (void) download : (NSURLDownload *) dl didFailWithError : (NSError *) err
{
	printf("download failed: %s\n", [[err localizedDescription] cString]);
	exit(0);
}

- (void)printProgress
{
	int i, n=0, downloaded=0, percent;
	float speed, elapsed;
	char progress[32];
	struct timeval tv;
	for(i=0;i<segment;i++) {
		downloaded += [[arr objectAtIndex: i] printCurrentProgress];
	}
	gettimeofday(&tv, NULL);
	elapsed = tv.tv_sec+(double)tv.tv_usec/1000000.0f-beginTime;
	if(elapsed < 0.5) speed = 0;
	else speed = (float)downloaded/elapsed/1024;
	percent = (int)(100.0*downloaded/length);
	progress[n++] = '|';
	for(i=0;i<20;i++) {
		if(percent/5 > i)
			progress[n++] = '=';
		else if(percent/5 == i)
			progress[n++] = '>';
		else progress[n++] ='-';
	}
	progress[n++] = '|';
	progress[n++] = '\0';
	mvprintw(segment+3, 0, "------  -------------------------------------------------------  ------------");
	mvprintw(segment+4, 0, "Total:  %s %3d%% [%d/%d]", progress, percent, downloaded, length);
	mvprintw(segment+4, 65, "%6.1f KB/s", speed);
	refresh();
}

@end